/******************************************************************************\
 *                        ____            _ _ 
 *                       |  _ \ ___ _ __ | (_) ___ __ _
 *                       | |_) / _ \ '_ \| | |/ __/ _` |
 *                       |  _ <  __/ |_) | | | (_| (_| |
 *                       |_| \_\___| .__/|_|_|\___\__,_|
 *                                 |_|
 ******************************************************************************
 *                                 
 *  Copyright (c) 2005-2006, Herbert Nachtnebel
 *                      
 ******************************************************************************
 *                      
 *  Title:    Replica - File Replication and Backup Utility
 *
 *  Version:  0.1
 *
 *  Date:     03-Dec-2005
 *
 *  Author:   Herbert Nachtnebel (Berri)
 *  
 ******************************************************************************
 *  
 *  Change log:
 *  
 *     Date     Rev  Ver                    Description
 *  ----------- --- ----- ---------------------------------------------------
 *  03-Dec-2005 Ber  0.1  Initial implementation
 *  
 ******************************************************************************
 *  
 *  Description:
 *
 *    Some macros to adapt the source to different machine architectures.
 *    
\******************************************************************************/
#ifndef COMPAT_H_INCLUDED
#define COMPAT_H_INCLUDED

#include <stdint.h>
#include <endian.h>
#include <byteswap.h>

#if __BYTE_ORDER == __LITTLE_ENDIAN

#define CACHE_SIGNATURE         CACHE_SIGNATURE_LE
#define SWAPPED_CACHE_SIGNATURE CACHE_SIGNATURE_BE

#define cpu2host_16(obj) bswap_16(obj)
#define cpu2host_32(obj) bswap_32(obj)
#define cpu2host_64(obj) bswap_64(obj)

#define host2cpu_16(obj) bswap_16(obj)
#define host2cpu_32(obj) bswap_32(obj)
#define host2cpu_64(obj) bswap_64(obj)

#elif __BYTE_ORDER == __BIG_ENDIAN

#define CACHE_SIGNATURE         CACHE_SIGNATURE_BE
#define SWAPPED_CACHE_SIGNATURE CACHE_SIGNATURE_LE

#define cpu2host_16(obj) (obj)
#define cpu2host_32(obj) (obj)
#define cpu2host_64(obj) (obj)

#define host2cpu_16(obj) (obj)
#define host2cpu_32(obj) (obj)
#define host2cpu_64(obj) (obj)

#else
#error "Machine byte order not supported"
#endif

#endif /* COMPAT_H_INCLUDED */
