/******************************************************************************\
 *                        ____            _ _ 
 *                       |  _ \ ___ _ __ | (_) ___ __ _
 *                       | |_) / _ \ '_ \| | |/ __/ _` |
 *                       |  _ <  __/ |_) | | | (_| (_| |
 *                       |_| \_\___| .__/|_|_|\___\__,_|
 *                                 |_|
 ******************************************************************************
 *                                 
 *  Copyright (c) 2005-2006, Herbert Nachtnebel
 *                      
 ******************************************************************************
 *                      
 *  Title:    Replica - File Replication and Backup Utility
 *
 *  Version:  0.1
 *
 *  Date:     10-Mar-2006
 *
 *  Author:   Herbert Nachtnebel (Berri)
 *  
 ******************************************************************************
 *  
 *  Change log:
 *  
 *     Date     Rev  Ver                    Description
 *  ----------- --- ----- ---------------------------------------------------
 *  10-Mar-2006 Ber  0.1  Initial implementation
 *  
 ******************************************************************************
 *  
 *  Description:
 *
 *    Tokens are strings stored uniquely in a binary sorted array.  Since
 *    they are unique, the pointer returned is a direct represention of the
 *    content and can be used to optimize comparisions or to avoid too
 *    complicated memory management.  Think of it as the hash of the content.
 *    
\******************************************************************************/
#ifndef TOKEN_H_INCLUDED
#define TOKEN_H_INCLUDED

/*
 *  Some macros to place objects into specific sections.
 */
#define TOKEN const token __attribute__((section(".token")))
#define INIT_FUNCTION(f) void (*f##_ptr)(void) __attribute__((section(".ctors"))) = &f;

/*
 *  The incredible complicated interface.
 */
typedef const char *token;
token unique (const char *s);

/*
 *  Initial tokens used in this application.
 */
extern TOKEN T_BLOB;
extern TOKEN T_COMMIT;
extern TOKEN T_ORIGIN;
extern TOKEN T_PARENT;
extern TOKEN T_POSIX;
extern TOKEN T_TREE;
extern TOKEN T_UTF_16BE;
extern TOKEN T_UTF_8;

#endif /* TOKEN_H_INCLUDED */
