/******************************************************************************\
 *                        ____            _ _ 
 *                       |  _ \ ___ _ __ | (_) ___ __ _
 *                       | |_) / _ \ '_ \| | |/ __/ _` |
 *                       |  _ <  __/ |_) | | | (_| (_| |
 *                       |_| \_\___| .__/|_|_|\___\__,_|
 *                                 |_|
 ******************************************************************************
 *                                 
 *  Copyright (c) 2005, Herbert Nachtnebel
 *
 *    based on code from the GIT project at your favorite kernel mirror [1]
 *
 *  Copyright (c) 2005, Linus Torvalds
 *                      Junio C Hamano
 *                      et al.
 *
 *  [1] http://www.kernel.org/pub/software/scm
 *  
 ******************************************************************************
 *  
 *  Title:    Replica - File Replication and Backup Utility
 *
 *  Version:  0.1
 *
 *  Date:     01-Nov-2005
 *
 *  Author:   Herbert Nachtnebel (Berri)
 *  
 ******************************************************************************
 *  
 *  Change log:
 *  
 *     Date     Rev  Ver                    Description
 *  ----------- --- ----- ---------------------------------------------------
 *  01-Nov-2005 Ber  0.1  Blatantly based (stolen) on git/{read,write}-tree.c
 *  
 ******************************************************************************
 *  
 *  Description:
 *
 *    Tree Operations
 *    ---------------
 *    
\******************************************************************************/
#ifndef TREE_H_INCLUDED
#define TREE_H_INCLUDED

/*
 *  Forward declare structures to avoid include hell.
 */
struct config;
struct dindex;

/*
 *  Posix metadata informations to regenerate a file.
 *
 *  Note!  All fields are in network byte order!
 *  
 *         This is a variable length structure where the name field uses
 *         only the required space (including the terminating zero byte/
 *         word).
 */
struct tree_entry 
{
    uint64_t te_ctime;                  /* status change time (nsec's) */
    uint64_t te_mtime;                  /* modification time  (nsec's) */
    uint32_t te_mode;                   /* protection mode, file type */
    uint32_t te_uid;                    /* owner user id */
    uint32_t te_gid;                    /* owner group id */
    uint8_t  te_sha1 [20];              /* SHA1 hash of data */
    char     te_name [0];               /* name of entry, variable length! */
};

/*
 *  Note!  The name uses UTF-8 or UTF-16BE encoding of Unicode characters,
 *         depending how the application was compiled.  Most western area
 *         countries will use the UTF-8 support, whereas most eastern area
 *         countries are better handled by using the UTF-16BE coding.
 */
#define S_UTF16BE      00004000000      /* UTF-16BE coding used for te_name */

/*
 *  Write out the index file in form of tree objects.
 */
int write_tree (struct config *config, struct dindex *dindex, uint8_t *sha1);

/*
 *  Write a commit object for our current index file state.
 */
int commit_tree (struct config *config, uint8_t *sha1, const uint8_t **parents);

#endif /* TREE_H_INCLUDED */
