/******************************************************************************\
 *                        ____            _ _ 
 *                       |  _ \ ___ _ __ | (_) ___ __ _
 *                       | |_) / _ \ '_ \| | |/ __/ _` |
 *                       |  _ <  __/ |_) | | | (_| (_| |
 *                       |_| \_\___| .__/|_|_|\___\__,_|
 *                                 |_|
 ******************************************************************************
 *                                 
 *  Copyright (c) 2005-2006, Herbert Nachtnebel
 *
 *    based on code from the GIT project at your favorite kernel mirror [1]
 *
 *  Copyright (c) 2005-2006, Linus Torvalds
 *                           Junio C Hamano
 *                           et al.
 *
 *  [1] http://www.kernel.org/pub/software/scm
 *  
 ******************************************************************************
 *                      
 *  Title:    Replica - File Replication and Backup Utility
 *
 *  Version:  0.1
 *
 *  Date:     01-Nov-2005
 *
 *  Author:   Herbert Nachtnebel (Berri)
 *  
 ******************************************************************************
 *  
 *  Change log:
 *  
 *     Date     Rev  Ver                    Description
 *  ----------- --- ----- ---------------------------------------------------
 *  01-Nov-2005 Ber  0.1  Initial implementation
 *  
 ******************************************************************************
 *  
 *  Description:
 *
 *    The Toolset
 *    -----------
 *    
\******************************************************************************/
#ifndef UPDATE_H_INCLUDED
#define UPDATE_H_INCLUDED

/*
 *  Forward declare structure to avoid include hell.
 */
struct stat;

/*
 *  Updates the metadata in the cache entry and returns what has changed.
 */
int update_metadata (struct cache_entry *ce, struct stat *st);

#define METADATA_CHANGED  1
#define FILEDATA_CHANGED  2

/*
 *  Reads the file and writes the blob to the database.
 */
int update_filedata (struct cache_entry *ce, const char *file, const char *dbase);

/*
 *  Write the given data buffer to a sha1 file in our database.
 */
int write_sha1_file (const char *dbase, token type, token subtype,
                     const void *buffer, size_t size, uint8_t *sha1);

/*
 *  Read a sha1 file from our database into a internal data buffer.
 */
int read_sha1_file (const char *dbase, const uint8_t *sha1, size_t limit,
                    token *type, token *subtype, size_t *size, void **buffer);

/******************************************************************************/

/*
 *  Dump the header of an object in our database.
 */
void dump_type_of (const char *dbase, const char *sha1);

/*
 *  Dump the contents of an object in our database.
 */
void dump_sha1_cat (const char *dbase, const char *sha1);


#endif /* UPDATE_H_INCLUDED */
